/*  binder.h
 *
 *  author: Eros Pedrini
 *  version: 1.0    (@2008)
 *
 *  STL extension for support binding of functions/method with two
 *  arguments.
 */

#if !defined BINDER_H__
#define BINDER_H__

#include <functional>

using namespace std;

namespace ep_tools {

// ternary function, for design compatibilty with STL
template<class _Arg1, class _Arg2, class _Arg3, class _Result>
struct ternary_function {
	typedef _Arg1 first_argument_type;
	typedef _Arg2 second_argument_type;
	typedef _Arg2 third_argument_type;
	typedef _Result result_type;
};

// mem_fun for methods with 2 input arguments
template<class _Result, class _Ty, class _Arg1, class _Arg2>
class mem_fun2_t : public ternary_function<_Ty *, _Arg1, _Arg2, _Result> {
public:
	explicit mem_fun2_t(_Result (_Ty::*_Pm)(_Arg1, _Arg2)) : _Pmemfun(_Pm) {}

	_Result operator()(_Ty *_Pleft, _Arg1 _One, _Arg2 _Two) const {
		return ((_Pleft->*_Pmemfun)(_One, _Two));
	}

private:
	_Result (_Ty::*_Pmemfun)(_Arg1, _Arg2);	// the member function pointer
};

template<class _Result, class _Ty, class _Arg1, class _Arg2> inline
mem_fun2_t<_Result, _Ty, _Arg1, _Arg2> mem_fun( _Result (_Ty::*_Pm)(_Arg1, _Arg2) ) {
	return (mem_fun2_t<_Result, _Ty, _Arg1, _Arg2>(_Pm));
}

// binder for functions with 2 input arguments
template<class _Fn3>
class binder2argfn : public binary_function<
			typename _Fn3::second_argument_type,
			typename _Fn3::third_argument_type,
			typename _Fn3::result_type> {
public:
	typedef binary_function<
			typename _Fn3::second_argument_type,
			typename _Fn3::third_argument_type,
			typename _Fn3::result_type> _Base;
	typedef typename _Base::first_argument_type first_argument_type;
	typedef typename _Base::second_argument_type second_argument_type;
	typedef typename _Base::result_type result_type;

	binder2argfn(const _Fn3& _Func, const typename _Fn3::first_argument_type& _Left) : op(_Func), value(_Left) {}

	result_type operator()(const first_argument_type& _Arg1, const second_argument_type& _Arg2) const {
		return (op(value, _Arg1, _Arg2));
	}

	result_type operator()(first_argument_type& _Arg1, second_argument_type& _Arg2) const {
		return (op(value, _Arg1, _Arg2));
	}

protected:
	_Fn3 op;	// the functor to apply
	typename _Fn3::first_argument_type value;	// the left operand
};


template<class _Fn3, class _Ty> inline
binder2argfn<_Fn3> bind2argfn(const _Fn3& _Func, const _Ty& _Left) {
	typename _Fn3::first_argument_type _Val(_Left);

	return (binder2argfn<_Fn3>(_Func, _Val));
}

}

#endif // BINDER_H__